//
//  Tag.swift
//  
//
//  Created by Thibaut Schmitt on 10/01/2022.
//

import Foundation
import ToolCore
import ArgumentParser

struct Tags: ParsableCommand {
    
    // MARK: - Command Configuration
    
    static var configuration = CommandConfiguration(
        abstract: "Generate tags extension file.",
        version: "0.1.0"
    )
    
    
    // MARK: - Static
    
    static let toolName = "Tags"
    static let defaultExtensionName = "Tags"
    static let noTranslationTag: String = "notranslation"
    
    // MARK: - Properties
    
    var extensionFileName: String {
        if options.extensionSuffix.isEmpty == false {
            return "\(options.extensionName)+\(options.extensionSuffix).swift"
        }
        return "\(options.extensionName).swift"
    }
    var extensionFilePath: String { "\(options.extensionOutputPath)/\(extensionFileName)" }
    var generateStaticVariable: Bool {
        options.extensionName == Self.defaultExtensionName
    }
    
    // MARK: - Command Options
    
    // The `@OptionGroup` attribute includes the flags, options, and
    // arguments defined by another `ParsableArguments` type.
    @OptionGroup var options: TagsOptions
    
    // MARK: - Run
    
    mutating func run() {
        print("[\(Self.toolName)] Starting tagss generation")
        
        // Check requirements
        guard checkRequirements() else { return }
        
        print("[\(Self.toolName)] Will generate tags")
        
        // Parse input file
        let sections = TwineFileParser.parse(options.inputFile)
        
        // Generate extension
        TagsGenerator.writeExtensionFiles(sections: sections,
                                          lang: options.lang,
                                          tags: ["ios", "iosonly", Self.noTranslationTag],
                                          staticVar: generateStaticVariable,
                                          extensionName: options.extensionName,
                                          extensionFilePath: extensionFilePath)
        
        print("[\(Self.toolName)] Tags generated")
    }
    
    // MARK: - Requirements
    
    private func checkRequirements() -> Bool {
        let fileManager = FileManager()
        
        // Input file
        guard fileManager.fileExists(atPath: options.inputFile) else {
            let error = StringiumError.fileNotExists(options.inputFile)
            print(error.localizedDescription)
            Stringium.exit(withError: error)
        }
        
        // Check if needed to regenerate
        guard GeneratorChecker.shouldGenerate(force: options.forceGeneration, inputFilePath: options.inputFile, extensionFilePath: extensionFilePath) else {
            print("[\(Self.toolName)] Tags are already up to date :) ")
            return false
        }
        
        return true
    }
}
